/*
 *  Copyright(c)1998 Forward Computing and Control Pty. Ltd.
 *  ACN 003 669 994, NSW, Australia     All rights Reserved
 *  
 *   Written by Dr. M.P. Ford
Logging package
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 
 */
/* 
 * Version 1.2
 * 22/10/04 added rolling of the System.err/System.out log
 */
 
package au.com.forward.logging;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 *  Redirects System.Err and optionally System.Out to an output file This 
 *  catches any uncaught exception traces. A heading can be written to the top 
 *  of the file. Autoflush is set so ensure last error saved to disk Each 
 *  application invocation appends to the log file.
 *  
 *  Use this class is you are using Log4J or some other non-Java logging package.
 *  <br>If you are using standard Java logging then consider using 
 * {@link Logging#initializeLogging(String, String ,boolean, int)} instead 
 * 
 *  To initialize this class add the following code to your main application class
<pre><code> 
 import au.com.forward.logging.*;
 
 static {
    // put all log files in user's home dir  
    String userHome = System.getProperty("user.home",".")
                      +System.getProperty("file.separator","/");
    // use current dir if cannot find user home 
    // set up logging for errors etc
    // throws RuntimeException if cannot open log file
    LogStdStreams.initializeErrorLogging(userHome+"applicationLog.log", 
                            "Log File for Application "+ new Date(),
                            true, true); 
                            // redirect System.out as well as err and append to existing log
 }                                        
</code></pre>
<p>
 *
 * Then in the body of the code any statements like
 * 
<pre><code> 
 System.err.println(" Error message ..");
 System.out.println(" Output ... ");
</code></pre>

 *
 * will be sent to the log file and not sent to the terminal window. 
 *
 * @author   Dr. M.P. Ford 
 * @created    October 21, 2001 
 * @version    1 5th Oct 2001 
 */

public class LogStdStreams extends PrintStream {

  /**
   *  The name of the output log file 
   */
  private static String logFileName;

//  private static SimpleDateFormat dateFormater = new SimpleDateFormat("yyyy_MM_dd");
  private static String initialString = "";
  /**
   *  The only instance of this class 
   */
  private static LogStdStreams logStream = null;

  /**
   *  True if System.Out is being redirected 
   */
  private static boolean redirectSystemOut = true;

  /**
   * Extension to add when rolling over log file
   */
  private static String BACKUP_EXT = ".old";

  /**
   *  Private constructor to create PrintStream an redirect standard streams 
   *
   * @param  logFile    the file output stream to use 
   * @param  logStdOut  true if the System.out to be redirected also 
   */
  private LogStdStreams(FileOutputStream logFile, boolean logStdOut) {
    super(logFile, true);
    // autoflush
    System.setErr(this);
    redirectSystemOut = logStdOut;
    if (redirectSystemOut) {
      // redirect stdOut aswell
      System.setOut(this);
    }
  }


  /**
   *  Get the PrintStream being used for this logfile 
   *
   * @return    the PrintStream System.err is being redirected to 
   * @throws    RuntimeException if initializeErrorLogging() has not been called 
   *      yet 
   */
  public static PrintStream getLogStream() {
    if (logStream == null) {
      throw new RuntimeException("initializeErrorLogging() has not been called yet.");
    }
    return logStream;
  }

  /**
   *  Has the logging already been initialized? 
   *
   * @return    true if logging already initialized else false 
   */
  public static boolean isLoggingInitialized() {
    return !(logStream == null);
  }

	
  /**
   *  Is the System.Out being redirected to this logfile 
   *
   * @return    true if System.Out is being redirected 
   * @throws    RuntimeException if initializeErrorLogging() has not been called 
   *      yet 
   */
  public static boolean isSystemOutRedirected() {
    if (logStream == null) {
      throw new RuntimeException("initializeErrorLogging() has not been called yet.");
    }
    return redirectSystemOut;
  }


  /**
   *  Get the name of the logfile 
   *
   * @return    true if System.Out is being redirected 
   * @throws    RuntimeException if initializeErrorLogging() has not been called 
   *      yet 
   */
  public static String getLogFileName() {
    if (logStream == null) {
      throw new RuntimeException("initializeErrorLogging() has not been called yet.");
    }
    return logFileName;
  }


  /**
   *  The static initialization method <br>
   *  Also redirects System.out, rewrites output file 
   *
   * @param  fileName  the name of the log file 
   * @throws           RuntimeException if initializeErrorLogging() has already 
   *      been called yet 
   */

  public static void initializeErrorLogging(String fileName) {
    initializeErrorLogging(fileName, null, true, false);
  }


  /**
   *  The static initialization method which writes heading <br>
   *  Also redirects System.out, rewrites output file 
   *
   * @param  fileName    the name of the log file 
   * @param  initialStr  the heading string to write to the log file when opened 
   * @throws             RuntimeException if initializeErrorLogging() has already 
   *      been called 
   */
  public static void initializeErrorLogging(String fileName, String initialStr) {
    initializeErrorLogging(fileName, initialStr, true, false);
  }


  /**
   *  The static initialization method which writes heading <br>
   *  Rewrites output file 
   *
   * @param  fileName    the name of the log file 
   * @param  initialStr  the heading string to write to the log file when opened 
   * @param  logStdOut   true if System.Out to redirected to log file also 
   * @throws             RuntimeException if initializeErrorLogging() has already 
   *      been called 
   */
  public static void initializeErrorLogging(String fileName, String initialStr, 
      boolean logStdOut) {
    initializeErrorLogging(fileName, initialStr, logStdOut, false);  // v1.1 was true,false);
  }


  /**
   *  The static initialization method 
   *
   * @param  fileName    the name of the log file 
   * @param  initialStr  the heading string to write to the log file when opened 
   * @param  logStdOut   true if System.Out to redirected to log file also 
   * @param  append      true if to append to existing log, false to rewrite new 
   *      log. 
   * @throws             RuntimeException if initializeErrorLogging() has already 
   *      been called or cannot create log file dir or open log file 
   */
  public static void initializeErrorLogging(String fileName, String initialStr, 
      boolean logStdOut, boolean append) {
    if (logStream != null) {
      throw new RuntimeException("initializeErrorLogging() has already been called.");
    }
    initialString = initialStr;
    File logDirFile = new File(fileName).getParentFile();
    if ((logDirFile != null) && (!logDirFile.exists())) {
      //try and create it
      if (!logDirFile.mkdirs()) {
        throw new RuntimeException("Error: Could not create log directory\n" + logDirFile);
      } 
    } 
    
    logFileName = fileName;
    try {
      logStream = new LogStdStreams(new FileOutputStream(logFileName, append), 
          logStdOut);
      // append
    } catch (Throwable tex) {
      throw new RuntimeException("Could not open output file " + logFileName +" "+tex.getMessage());
    }
    if ((initialStr != null) && (initialStr.length() != 0)) {
      // write heading
      System.err.println(initialStr);
    }
  }

  /**
   *  The static initialization method to roll logfile.
   * <br>Will roll log file to .old when it exceeds the specified Kbytes
   *
   * @param  fileName    the name of the log file 
   * @param  initialStr  the heading string to write to the log file when opened 
   * @param  logStdOut   true if System.Out to redirected to log file also 
   * @param  maxSizeInKbytes  roll current file to .old if it exceeds this size on
   * startup.  Any existing .old log file is deleted. 
   * @throws             RuntimeException if initializeErrorLogging() has already 
   *      been called or cannot create log file dir or open log file 
   */
  public static void initializeErrorLogging(String fileName, String initialStr, 
          boolean logStdOut, int maxSizeInKbytes) {
      
      if (maxSizeInKbytes < 0) {
          initializeErrorLogging(fileName, initialStr,logStdOut, true);
          System.err.println("Error maxSizeInMeg must not be negative.");
          return;
      }
      if (maxSizeInKbytes > 0) {
          // try and roll
          long length = (new File(fileName)).length();
          if (length > maxSizeInKbytes*1024) {
              // roll it
              // remove existing backup
              File backupFile = new File(fileName+BACKUP_EXT); 
              backupFile.delete();
              // now try and rename extisting logfile
              if (!(new File(fileName).renameTo(backupFile))) {
                  initializeErrorLogging(fileName, initialStr,logStdOut, true);
                  System.err.println("Error could not rename to log file to '"+backupFile+"'");
                  return;                  
              }
          }
      }
      // ok now have rolled if needed open logfile here
      initializeErrorLogging(fileName, initialStr,logStdOut, true);
  }
      
  
  /**
   *  A test main() 
   *
   * @param  args  Description of Parameter 
   */
  public static void main(String args[]) {
      initializeErrorLogging("test.log","Test of file rolling " + new Date(),true,0);
    // test call to initializeErrorLogging again
    try {
      initializeErrorLogging("test.log");
    } catch (Exception ex) {
      // test redirect of System.out
      System.out.println(ex.toString());
    }
    // check logfile and isSystemOutRedirected()
    System.out.println("Log file name is " + getLogFileName());
    System.out.println("is System.Out redirected? " + isSystemOutRedirected());
  }

}